import 'dart:io';

import 'package:dio/dio.dart';
import 'package:xh_framework/com/xh/networkapi/config/network_config.dart';
import 'package:xh_framework/com/xh/networkapi/exception/network_exception.dart';

typedef OnSuccessCallback<T> = void Function(T data);
typedef OnErrorCallback = void Function(NetworkException e);
typedef OnProgressCallback = void Function(int count, int total);

const OK = 200;

abstract class HttpRequest {
  void getRequest<T>(String url, Map<String, dynamic> map,
      {OnSuccessCallback<T> onSuccess, OnErrorCallback onError});

  void postRequest<T>(String url, Map<String, dynamic> map,
      {OnSuccessCallback<T> onSuccess, OnErrorCallback onError});

  void uploadRequest<T>(String url, Map<String, dynamic> map,
      {OnSuccessCallback<T> onSuccess, OnErrorCallback onError});

  void downloadRequest(String url, String path,
      {OnSuccessCallback<String> onSuccess,
      OnProgressCallback onProgress,
      OnErrorCallback onError});
}

class XHHttpRequest implements HttpRequest {
  final Dio dio = new Dio();

  @override
  void getRequest<T>(String url, Map<String, dynamic> map,
      {OnSuccessCallback<T> onSuccess, OnErrorCallback onError}) async {
    try {
      Options options = new Options(
        connectTimeout: NetworkConfig.getInstance().getConnectTimeout(),
        sendTimeout: NetworkConfig.getInstance().getWriteTimeout(),
        receiveTimeout: NetworkConfig.getInstance().getReadTimeout(),
      );
      Response<T> response =
          await dio.get<T>(url, queryParameters: map, options: options);
      if (response.statusCode == OK) {
        onSuccess(response.data);
      }
    } catch (e) {
      onErrorHandler(e, onError);
    }
  }

  @override
  void postRequest<T>(String url, Map<String, dynamic> map,
      {OnSuccessCallback<T> onSuccess, OnErrorCallback onError}) async {
    try {
      Response response = await dio.post<T>(url, queryParameters: map);
      if (response.statusCode == OK) {
        onSuccess(response.data);
      }
    } catch (e) {
      onErrorHandler(e, onError);
    }
  }

  @override
  void uploadRequest<T>(String url, Map<String, dynamic> map,
      {OnSuccessCallback<T> onSuccess, OnErrorCallback onError}) async {
    try {
      FormData data = FormData();
      map.forEach((String key, dynamic value) {
        if (value is File) {
          File file = value;
          String fileName = file.path.substring(file.path.lastIndexOf("/") + 1);
          UploadFileInfo fileInfo = new UploadFileInfo(value, fileName);
          data.add(key, fileInfo);
        } else {
          data.add(key, value);
        }
      });

      Options options = Options(connectTimeout: 3000, sendTimeout: 15000);

      Response response = await dio.post<T>(url, options: options, data: data);
      if (response.statusCode == OK) {
        onSuccess(response.data);
      }
    } catch (e) {
      onErrorHandler(e, onError);
    }
  }

  @override
  void downloadRequest(String url, String path,
      {OnSuccessCallback<String> onSuccess,
      OnProgressCallback onProgress,
      OnErrorCallback onError}) async {
    try {
      Response response = await dio.download(url, path,
          onReceiveProgress: (count, total) => onProgress(count, total));
      if (response.statusCode == OK) {
        onSuccess(path);
      }
    } catch (e) {
      onErrorHandler(e, onError);
    }
  }

  void onErrorHandler(e, OnErrorCallback onError) {
    NetworkException exception;
    if (e is DioError) {
      if (e.type == DioErrorType.CONNECT_TIMEOUT) {
        exception = NetworkException(
            action: NetworkException.connectionTimeout,
            message: NetworkException.connectionTimeoutMsg);
      } else if (e.type == DioErrorType.SEND_TIMEOUT) {
        exception = NetworkException(
            action: NetworkException.writeTimeout,
            message: NetworkException.writeTimeoutMsg);
      } else if (e.type == DioErrorType.RECEIVE_TIMEOUT) {
        exception = NetworkException(
            action: NetworkException.readTimeout,
            message: NetworkException.readTimeoutMsg);
      } else if (e.type == DioErrorType.RESPONSE) {
        exception = NetworkException(
            action: NetworkException.serverError,
            message: NetworkException.serverErrorMsg);
      } else if (e.type == DioErrorType.DEFAULT) {
        exception = NetworkException(
            action: NetworkException.clientError,
            message: NetworkException.clientErrorMsg);
      } else if (e.type == DioErrorType.CANCEL) {
        exception = NetworkException(
            action: NetworkException.cancelError,
            message: NetworkException.cancelErrorMsg);
      } else {
        exception = NetworkException(
            action: NetworkException.otherError,
            message: NetworkException.otherErrorMsg);
      }
    } else if (e is FileSystemException) {
      exception = NetworkException(
          action: NetworkException.fileError,
          message: e.message.toString() +
              (e.osError != null ? ":" + e.osError.message : ""));
    } else {
      exception = NetworkException(
          action: NetworkException.otherError,
          message: NetworkException.otherErrorMsg);
    }
    if (onError != null) {
      onError(exception);
    }
  }
}
